@@ -1,6 +1,7 @@
+import warnings
 import logging
-import time
 import os
+import time
 
 
 def get_logger(name):
@@ -31,6 +32,29 @@ def get_logger(name):
     return logger
 
 
+def raise_deprecation_warning(
+    message: str = "",
+    logger: logging.Logger = get_logger("main_logger"),
+):
+    """
+    Raises a DeprecationWarning.
+
+    Parameters
+    ----------
+    message
+        The message of the ValueError.
+    logger
+        The logger instance to log the error message if 'condition' is False.
+
+    Raises
+    ------
+    Warning
+        DeprecationWarning
+    """
+
+    logger.warning("DeprecationWarning: " + message)
+
+
 def raise_if_not(
     condition: bool,
     message: str = "",
@@ -130,9 +154,7 @@ def time_log(logger: logging.Logger = get_logger("main_logger")):
             result = method(*args, **kwargs)
             end_time = time.time()
             duration = int((end_time - start_time) * 1000)
-            logger.info(
-                method.__name__ + " function ran for {} milliseconds".format(duration)
-            )
+            logger.info(method.__name__ + f" function ran for {duration} milliseconds")
             return result
 
         return timed
@@ -140,7 +162,7 @@ def time_log(logger: logging.Logger = get_logger("main_logger")):
     return time_log_helper
 
 
-class SuppressStdoutStderr(object):
+class SuppressStdoutStderr:
     """
     A context manager for doing a "deep suppression" of stdout and stderr in
     Python, i.e. will suppress all print, even if the print originates in a
@@ -192,3 +214,17 @@ def execute_and_suppress_output(function, logger, suppression_threshold_level, *
     else:
         return_value = function(*args)
     return return_value
+
+
+def suppress_lightning_warnings(suppress_all: bool = False):
+    warnings.filterwarnings(
+        "ignore", ".*You defined a `validation_step` but have no `val_dataloader`.*"
+    )
+    if suppress_all:
+        warnings.filterwarnings(
+            "ignore", ".*does not have many workers which may be a bottleneck.*"
+        )
+        warnings.filterwarnings(
+            "ignore",
+            ".*Trying to infer the `batch_size` from an ambiguous collection.*",
+        )
