@@ -10,13 +10,14 @@ References
 .. [1] https://wikipedia.org/wiki/Autoregressive_integrated_moving_average
 """
 
-from statsmodels.tsa.arima.model import ARIMA as staARIMA
 from typing import Optional, Tuple
-import numpy as np
 
+import numpy as np
+from statsmodels.tsa.arima.model import ARIMA as staARIMA
+
+from darts.logging import get_logger
 from darts.models.forecasting.forecasting_model import DualCovariatesForecastingModel
 from darts.timeseries import TimeSeries
-from darts.logging import get_logger
 
 logger = get_logger(__name__)
 
@@ -64,9 +65,7 @@ class ARIMA(DualCovariatesForecastingModel):
             return f"ARIMA{self.order}"
         return f"SARIMA{self.order}x{self.seasonal_order}"
 
-    def _fit(
-        self, series: TimeSeries, future_covariates: Optional[TimeSeries] = None
-    ) -> None:
+    def _fit(self, series: TimeSeries, future_covariates: Optional[TimeSeries] = None):
 
         super()._fit(series, future_covariates)
         m = staARIMA(
@@ -78,6 +77,8 @@ class ARIMA(DualCovariatesForecastingModel):
         )
         self.model = m.fit()
 
+        return self
+
     def _predict(
         self,
         n: int,
