@@ -5,10 +5,12 @@ Linear Regression model
 A forecasting model using a linear regression of some of the target series' lags, as well as optionally some
 covariate series' lags in order to obtain a forecast.
 """
-from typing import Union, Tuple, List
+from typing import List, Tuple, Union
+
+from sklearn.linear_model import LinearRegression
+
 from darts.logging import get_logger
 from darts.models.forecasting.regression_model import RegressionModel
-from sklearn.linear_model import LinearRegression
 
 logger = get_logger(__name__)
 
@@ -19,6 +21,7 @@ class LinearRegressionModel(RegressionModel):
         lags: Union[int, list] = None,
         lags_past_covariates: Union[int, List[int]] = None,
         lags_future_covariates: Union[Tuple[int, int], List[int]] = None,
+        output_chunk_length: int = 1,
         **kwargs,
     ):
         """Linear regression model.
@@ -37,6 +40,10 @@ class LinearRegressionModel(RegressionModel):
             given the last `past` lags in the past are used (inclusive, starting from lag -1) along with the first
             `future` future lags (starting from 0 - the prediction time - up to `future - 1` included). Otherwise a list
             of integers with lags is required.
+        output_chunk_length
+            Number of time steps predicted at once by the internal regression model. Does not have to equal the forecast
+            horizon `n` used in `predict()`. However, setting `output_chunk_length` equal to the forecast horizon may
+            be useful if the covariates don't extend far enough into the future.
         **kwargs
             Additional keyword arguments passed to `sklearn.linear_model.LinearRegression`.
         """
@@ -45,13 +52,9 @@ class LinearRegressionModel(RegressionModel):
             lags=lags,
             lags_past_covariates=lags_past_covariates,
             lags_future_covariates=lags_future_covariates,
+            output_chunk_length=output_chunk_length,
             model=LinearRegression(**kwargs),
         )
 
     def __str__(self):
-        return (
-            f"LinearRegression(lags={self.lags}, "
-            f"lags_past_covariates={self.lags_past_covariates}, "
-            f"lags_historical_covariates={self.lags_historical_covariates}, "
-            f"lags_future_covariates={self.lags_future_covariates})"
-        )
+        return f"LinearRegression(lags={self.lags})"
