@@ -3,26 +3,22 @@ Recurrent Neural Networks
 -------------------------
 """
 
-import torch.nn as nn
-import torch
-from numpy.random import RandomState
-from typing import Sequence, Optional, Union, Tuple
-from darts.timeseries import TimeSeries
+from typing import Optional, Sequence, Tuple, Union
 
-from darts.logging import raise_if_not, get_logger
-from darts.models.forecasting.torch_forecasting_model import (
-    TorchParametricProbabilisticForecastingModel,
-    DualCovariatesTorchModel,
-)
-from darts.utils.torch import random_method
+import torch
+import torch.nn as nn
+
+from darts.logging import get_logger, raise_if_not
+from darts.models.forecasting.pl_forecasting_module import PLDualCovariatesModule
+from darts.models.forecasting.torch_forecasting_model import DualCovariatesTorchModel
+from darts.timeseries import TimeSeries
 from darts.utils.data import DualCovariatesShiftedDataset, TrainingDataset
-from darts.utils.likelihood_models import Likelihood
 
 logger = get_logger(__name__)
 
 
 # TODO add batch norm
-class _RNNModule(nn.Module):
+class _RNNModule(PLDualCovariatesModule):
     def __init__(
         self,
         name: str,
@@ -32,6 +28,7 @@ class _RNNModule(nn.Module):
         target_size: int,
         nr_params: int,
         dropout: float = 0.0,
+        **kwargs
     ):
 
         """PyTorch module implementing an RNN to be used in `RNNModel`.
@@ -57,6 +54,8 @@ class _RNNModule(nn.Module):
             The number of parameters of the likelihood (or 1 if no likelihood is used).
         dropout
             The fraction of neurons that are dropped in all-but-last RNN layers.
+        **kwargs
+            all parameters required for :class:`darts.model.forecasting_models.PLForecastingModule` base class.
 
         Inputs
         ------
@@ -71,7 +70,11 @@ class _RNNModule(nn.Module):
             However, this module always returns the whole Tensor.
         """
 
-        super(_RNNModule, self).__init__()
+        # RNNModule doesn't really need input and output_chunk_length for PLModule
+        super().__init__(**kwargs)
+
+        # required for all modules -> saves hparams for checkpoints
+        self.save_hyperparameters()
 
         # Defining parameters
         self.target_size = target_size
@@ -102,217 +105,6 @@ class _RNNModule(nn.Module):
         # returns outputs for all inputs, only the last one is needed for prediction time
         return predictions, last_hidden_state
 
-
-class RNNModel(TorchParametricProbabilisticForecastingModel, DualCovariatesTorchModel):
-    @random_method
-    def __init__(
-        self,
-        model: Union[str, nn.Module] = "RNN",
-        input_chunk_length: int = 12,
-        hidden_dim: int = 25,
-        n_rnn_layers: int = 1,
-        dropout: float = 0.0,
-        training_length: int = 24,
-        likelihood: Optional[Likelihood] = None,
-        random_state: Optional[Union[int, RandomState]] = None,
-        **kwargs
-    ):
-
-        """Recurrent Neural Network Model (RNNs).
-
-        This class provides three variants of RNNs:
-
-        * Vanilla RNN
-
-        * LSTM
-
-        * GRU
-
-        RNNModel is fully recurrent in the sense that, at prediction time, an output is computed using these inputs:
-
-        - previous target value, which will be set to the last known target value for the first prediction,
-          and for all other predictions it will be set to the previous prediction (in an auto-regressive fashion),
-        - the previous hidden state,
-        - the covariates at time `t` for forecasting the target at time `t` (if the model was trained with covariates),
-
-        This model supports future covariates; and it requires these covariates to extend far enough in the past
-        and the future (it's a so-called "dual covariates" model as the future covariates have to be provided both
-        in the past and the future). The model will complain if the provided `future_covariates` series doesn't have
-        an appropriate time span.
-
-        For a block version using an RNN model as an encoder only and supporting past
-        covariates, checkout `BlockRNNModel`.
-
-        Parameters
-        ----------
-        model
-            Either a string specifying the RNN module type ("RNN", "LSTM" or "GRU"),
-            or a PyTorch module with the same specifications as
-            `darts.models.rnn_model._RNNModule`.
-        input_chunk_length
-            Number of past time steps that are fed to the forecasting module at prediction time.
-        hidden_dim
-            Size for feature maps for each hidden RNN layer (:math:`h_n`).
-        n_rnn_layers
-            The number of recurrent layers.
-        dropout
-            Fraction of neurons afected by Dropout.
-        training_length
-            The length of both input (target and covariates) and output (target) time series used during
-            training. Generally speaking, `training_length` should have a higher value than `input_chunk_length`
-            because otherwise during training the RNN is never run for as many iterations as it will during
-            training. For more information on this parameter, please see `darts.utils.data.ShiftedDataset`
-        likelihood
-            Optionally, the likelihood model to be used for probabilistic forecasts.
-            If no likelihood model is provided, forecasts will be deterministic.
-        random_state
-            Control the randomness of the weights initialization. Check this
-            `link <https://scikit-learn.org/stable/glossary.html#term-random_state>`_ for more details.
-
-        batch_size
-            Number of time series (input and output sequences) used in each training pass.
-        n_epochs
-            Number of epochs over which to train the model.
-        add_encoders
-            A large number of past and future covariates can be automatically generated with `add_encoders`.
-            This can be done by adding mutliple pre-defined index encoders and/or custom user-made functions that
-            will be used as index encoders. Additionally, a transformer such as Darts' :class:`Scaler` can be added to
-            transform the generated covariates. This happens all under one hood and only needs to be specified at
-            model creation.
-            Read :meth:`SequentialEncoder <darts.utils.data.encoders.SequentialEncoder>` to find out more about
-            `add_encoders`. An example showing some of `add_encoders` features:
-
-            .. highlight:: python
-            .. code-block:: python
-
-                add_encoders={
-                    'cyclic': {'future': ['month']},
-                    'datetime_attribute': {'future': ['hour', 'dayofweek']},
-                    'position': {'past': ['absolute'], 'future': ['relative']},
-                    'custom': {'past': [lambda idx: (idx.year - 1950) / 50]},
-                    'transformer': Scaler()
-                }
-            ..
-        optimizer_cls
-            The PyTorch optimizer class to be used (default: `torch.optim.Adam`).
-        optimizer_kwargs
-            Optionally, some keyword arguments for the PyTorch optimizer (e.g., ``{'lr': 1e-3}``
-            for specifying a learning rate). Otherwise the default values of the selected `optimizer_cls`
-            will be used.
-        lr_scheduler_cls
-            Optionally, the PyTorch learning rate scheduler class to be used. Specifying `None` corresponds
-            to using a constant learning rate.
-        lr_scheduler_kwargs
-            Optionally, some keyword arguments for the PyTorch optimizer.
-        loss_fn
-            PyTorch loss function used for training.
-            This parameter will be ignored for probabilistic models if the `likelihood` parameter is specified.
-            Default: ``torch.nn.MSELoss()``.
-        model_name
-            Name of the model. Used for creating checkpoints and saving tensorboard data. If not specified,
-            defaults to the following string ``"YYYY-mm-dd_HH:MM:SS_torch_model_run_PID"``, where the initial part of the
-            name is formatted with the local date and time, while PID is the processed ID (preventing models spawned at
-            the same time by different processes to share the same model_name). E.g.,
-            ``"2021-06-14_09:53:32_torch_model_run_44607"``.
-        work_dir
-            Path of the working directory, where to save checkpoints and Tensorboard summaries.
-            (default: current working directory).
-        log_tensorboard
-            If set, use Tensorboard to log the different parameters. The logs will be located in:
-            `[work_dir]/.darts/runs/`.
-        nr_epochs_val_period
-            Number of epochs to wait before evaluating the validation loss (if a validation
-            ``TimeSeries`` is passed to the :func:`fit()` method).
-        torch_device_str
-            Optionally, a string indicating the torch device to use. (default: "cuda:0" if a GPU
-            is available, otherwise "cpu")
-        force_reset
-            If set to `True`, any previously-existing model with the same name will be reset (all checkpoints will
-            be discarded).
-        save_checkpoints
-            Whether or not to automatically save the untrained model and checkpoints from training.
-            If set to `False`, the model can still be manually saved using :func:`save_model()`
-            and loaded using :func:`load_model()`.
-        """
-
-        kwargs["input_chunk_length"] = input_chunk_length
-        kwargs["output_chunk_length"] = 1
-        super().__init__(likelihood=likelihood, **kwargs)
-
-        # check we got right model type specified:
-        if model not in ["RNN", "LSTM", "GRU"]:
-            raise_if_not(
-                isinstance(model, nn.Module),
-                '{} is not a valid RNN model.\n Please specify "RNN", "LSTM", '
-                '"GRU", or give your own PyTorch nn.Module'.format(
-                    model.__class__.__name__
-                ),
-                logger,
-            )
-
-        self.rnn_type_or_module = model
-        self.dropout = dropout
-        self.hidden_dim = hidden_dim
-        self.n_rnn_layers = n_rnn_layers
-        self.training_length = training_length
-
-    def _create_model(self, train_sample: Tuple[torch.Tensor]) -> torch.nn.Module:
-        # samples are made of (past_target, historic_future_covariates, future_covariates, future_target)
-        # historic_future_covariates and future_covariates have the same width
-        input_dim = train_sample[0].shape[1] + (
-            train_sample[1].shape[1] if train_sample[1] is not None else 0
-        )
-        output_dim = train_sample[-1].shape[1]
-        nr_params = 1 if self.likelihood is None else self.likelihood.num_parameters
-
-        if self.rnn_type_or_module in ["RNN", "LSTM", "GRU"]:
-            model = _RNNModule(
-                name=self.rnn_type_or_module,
-                input_size=input_dim,
-                target_size=output_dim,
-                nr_params=nr_params,
-                hidden_dim=self.hidden_dim,
-                dropout=self.dropout,
-                num_layers=self.n_rnn_layers,
-            )
-        else:
-            model = self.rnn_type_or_module(
-                name="custom_module",
-                input_size=input_dim,
-                target_size=output_dim,
-                nr_params=nr_params,
-                hidden_dim=self.hidden_dim,
-                dropout=self.dropout,
-                num_layers=self.n_rnn_layers,
-            )
-        return model
-
-    def _build_train_dataset(
-        self,
-        target: Sequence[TimeSeries],
-        past_covariates: Optional[Sequence[TimeSeries]],
-        future_covariates: Optional[Sequence[TimeSeries]],
-        max_samples_per_ts: Optional[int],
-    ) -> DualCovariatesShiftedDataset:
-
-        return DualCovariatesShiftedDataset(
-            target_series=target,
-            covariates=future_covariates,
-            length=self.training_length,
-            shift=1,
-            max_samples_per_ts=max_samples_per_ts,
-        )
-
-    def _verify_train_dataset_type(self, train_dataset: TrainingDataset):
-        raise_if_not(
-            isinstance(train_dataset, DualCovariatesShiftedDataset),
-            "RNNModel requires a training dataset of type DualCovariatesShiftedDataset.",
-        )
-        raise_if_not(
-            train_dataset.ds_past.shift == 1,
-            "RNNModel requires a shifted training dataset with shift=1.",
-        )
-
     def _produce_train_output(self, input_batch: Tuple):
         past_target, historic_future_covariates, future_covariates = input_batch
         # For the RNN we concatenate the past_target with the future_covariates
@@ -322,11 +114,11 @@ class RNNModel(TorchParametricProbabilisticForecastingModel, DualCovariatesTorch
             if future_covariates is not None
             else past_target
         )
-        return self.model(model_input)[0]
+        return self(model_input)[0]
 
-    @random_method
     def _produce_predict_output(self, x, last_hidden_state=None):
-        output, hidden = self.model(x, last_hidden_state)
+        """overwrite parent classes `_produce_predict_output` method"""
+        output, hidden = self(x, last_hidden_state)
         if self.likelihood:
             return self.likelihood.sample(output), hidden
         else:
@@ -388,3 +180,258 @@ class RNNModel(TorchParametricProbabilisticForecastingModel, DualCovariatesTorch
         batch_prediction = batch_prediction[:, :n, :]
 
         return batch_prediction
+
+
+class RNNModel(DualCovariatesTorchModel):
+    def __init__(
+        self,
+        input_chunk_length: int,
+        model: Union[str, nn.Module] = "RNN",
+        hidden_dim: int = 25,
+        n_rnn_layers: int = 1,
+        dropout: float = 0.0,
+        training_length: int = 24,
+        **kwargs
+    ):
+
+        """Recurrent Neural Network Model (RNNs).
+
+        This class provides three variants of RNNs:
+
+        * Vanilla RNN
+
+        * LSTM
+
+        * GRU
+
+        RNNModel is fully recurrent in the sense that, at prediction time, an output is computed using these inputs:
+
+        - previous target value, which will be set to the last known target value for the first prediction,
+          and for all other predictions it will be set to the previous prediction (in an auto-regressive fashion),
+        - the previous hidden state,
+        - the covariates at time `t` for forecasting the target at time `t` (if the model was trained with covariates),
+
+        This model supports future covariates; and it requires these covariates to extend far enough in the past
+        and the future (it's a so-called "dual covariates" model as the future covariates have to be provided both
+        in the past and the future). The model will complain if the provided `future_covariates` series doesn't have
+        an appropriate time span.
+
+        For a block version using an RNN model as an encoder only and supporting past
+        covariates, checkout `BlockRNNModel`.
+
+        Parameters
+        ----------
+        input_chunk_length
+            Number of past time steps that are fed to the forecasting module at prediction time.
+        model
+            Either a string specifying the RNN module type ("RNN", "LSTM" or "GRU"),
+            or a PyTorch module with the same specifications as
+            `darts.models.rnn_model._RNNModule`.
+        hidden_dim
+            Size for feature maps for each hidden RNN layer (:math:`h_n`).
+        n_rnn_layers
+            The number of recurrent layers.
+        dropout
+            Fraction of neurons afected by Dropout.
+        training_length
+            The length of both input (target and covariates) and output (target) time series used during
+            training. Generally speaking, `training_length` should have a higher value than `input_chunk_length`
+            because otherwise during training the RNN is never run for as many iterations as it will during
+            training. For more information on this parameter, please see `darts.utils.data.ShiftedDataset`
+        **kwargs
+            Optional arguments to initialize the pytorch_lightning.Module, pytorch_lightning.Trainer, and
+            Darts' :class:`TorchForecastingModel`.
+
+        loss_fn
+            PyTorch loss function used for training.
+            This parameter will be ignored for probabilistic models if the ``likelihood`` parameter is specified.
+            Default: ``torch.nn.MSELoss()``.
+        likelihood
+            The likelihood model to be used for probabilistic forecasts.
+        optimizer_cls
+            The PyTorch optimizer class to be used (default: ``torch.optim.Adam``).
+        optimizer_kwargs
+            Optionally, some keyword arguments for the PyTorch optimizer (e.g., ``{'lr': 1e-3}``
+            for specifying a learning rate). Otherwise the default values of the selected ``optimizer_cls``
+            will be used.
+        lr_scheduler_cls
+            Optionally, the PyTorch learning rate scheduler class to be used. Specifying ``None`` corresponds
+            to using a constant learning rate.
+        lr_scheduler_kwargs
+            Optionally, some keyword arguments for the PyTorch learning rate scheduler.
+        batch_size
+            Number of time series (input and output sequences) used in each training pass.
+        n_epochs
+            Number of epochs over which to train the model.
+        model_name
+            Name of the model. Used for creating checkpoints and saving tensorboard data. If not specified,
+            defaults to the following string ``"YYYY-mm-dd_HH:MM:SS_torch_model_run_PID"``, where the initial part
+            of the name is formatted with the local date and time, while PID is the processed ID (preventing models
+            spawned at the same time by different processes to share the same model_name). E.g.,
+            ``"2021-06-14_09:53:32_torch_model_run_44607"``.
+        work_dir
+            Path of the working directory, where to save checkpoints and Tensorboard summaries.
+            (default: current working directory).
+        log_tensorboard
+            If set, use Tensorboard to log the different parameters. The logs will be located in:
+            ``"{work_dir}/darts_logs/{model_name}/logs/"``.
+        nr_epochs_val_period
+            Number of epochs to wait before evaluating the validation loss (if a validation
+            ``TimeSeries`` is passed to the :func:`fit()` method).
+        torch_device_str
+            Optionally, a string indicating the torch device to use. (default: "cuda:0" if a GPU
+            is available, otherwise "cpu")
+        force_reset
+            If set to ``True``, any previously-existing model with the same name will be reset (all checkpoints will
+            be discarded).
+        save_checkpoints
+            Whether or not to automatically save the untrained model and checkpoints from training.
+            To load the model from checkpoint, call :func:`MyModelClass.load_from_checkpoint()`, where
+            :class:`MyModelClass` is the :class:`TorchForecastingModel` class that was used (such as :class:`TFTModel`,
+            :class:`NBEATSModel`, etc.). If set to ``False``, the model can still be manually saved using
+            :func:`save_model()` and loaded using :func:`load_model()`.
+        add_encoders
+            A large number of past and future covariates can be automatically generated with `add_encoders`.
+            This can be done by adding multiple pre-defined index encoders and/or custom user-made functions that
+            will be used as index encoders. Additionally, a transformer such as Darts' :class:`Scaler` can be added to
+            transform the generated covariates. This happens all under one hood and only needs to be specified at
+            model creation.
+            Read :meth:`SequentialEncoder <darts.utils.data.encoders.SequentialEncoder>` to find out more about
+            ``add_encoders``. An example showing some of ``add_encoders`` features:
+
+            .. highlight:: python
+            .. code-block:: python
+
+                add_encoders={
+                    'cyclic': {'future': ['month']},
+                    'datetime_attribute': {'future': ['hour', 'dayofweek']},
+                    'position': {'past': ['absolute'], 'future': ['relative']},
+                    'custom': {'past': [lambda idx: (idx.year - 1950) / 50]},
+                    'transformer': Scaler()
+                }
+            ..
+        random_state
+            Control the randomness of the weights initialization. Check this
+            `link <https://scikit-learn.org/stable/glossary.html#term-random_state>`_ for more details.
+        pl_trainer_kwargs
+            By default :class:`TorchForecastingModel` creates a PyTorch Lightning Trainer with several useful presets
+            that performs the training, validation and prediction processes. These presets include automatic
+            checkpointing, tensorboard logging, setting the torch device and more.
+            With ``pl_trainer_kwargs`` you can add additional kwargs to instantiate the PyTorch Lightning trainer
+            object. Check the `PL Trainer documentation
+            <https://pytorch-lightning.readthedocs.io/en/stable/common/trainer.html>`_ for more information about the
+            supported kwargs.
+            With parameter ``"callbacks"`` you can add custom or PyTorch-Lightning built-in callbacks to Darts'
+            :class:`TorchForecastingModel`. Below is an example for adding EarlyStopping to the training process.
+            The model will stop training early if the validation loss `val_loss` does not improve beyond
+            specifications. For more information on callbacks, visit:
+            `PyTorch Lightning Callbacks
+            <https://pytorch-lightning.readthedocs.io/en/stable/extensions/callbacks.html>`_
+
+            .. highlight:: python
+            .. code-block:: python
+
+                from pytorch_lightning.callbacks.early_stopping import EarlyStopping
+
+                # stop training when validation loss does not decrease more than 0.05 (`min_delta`) over
+                # a period of 5 epochs (`patience`)
+                my_stopper = EarlyStopping(
+                    monitor="val_loss",
+                    patience=5,
+                    min_delta=0.05,
+                    mode='min',
+                )
+
+                pl_trainer_kwargs={"callbacks": [my_stopper]}
+            ..
+
+            Note that you can also use a custom PyTorch Lightning Trainer for training and prediction with optional
+            parameter ``trainer`` in :func:`fit()` and :func:`predict()`.
+        show_warnings
+            whether to show warnings raised from PyTorch Lightning. Useful to detect potential issues of
+            your forecasting use case.
+        """
+        # create copy of model parameters
+        model_kwargs = {key: val for key, val in self.model_params.items()}
+        model_kwargs["output_chunk_length"] = 1
+
+        super().__init__(**self._extract_torch_model_params(**model_kwargs))
+
+        # extract pytorch lightning module kwargs
+        self.pl_module_params = self._extract_pl_module_params(**model_kwargs)
+
+        # check we got right model type specified:
+        if model not in ["RNN", "LSTM", "GRU"]:
+            raise_if_not(
+                isinstance(model, nn.Module),
+                '{} is not a valid RNN model.\n Please specify "RNN", "LSTM", '
+                '"GRU", or give your own PyTorch nn.Module'.format(
+                    model.__class__.__name__
+                ),
+                logger,
+            )
+
+        self.rnn_type_or_module = model
+        self.dropout = dropout
+        self.hidden_dim = hidden_dim
+        self.n_rnn_layers = n_rnn_layers
+        self.training_length = training_length
+
+    def _create_model(self, train_sample: Tuple[torch.Tensor]) -> torch.nn.Module:
+        # samples are made of (past_target, historic_future_covariates, future_covariates, future_target)
+        # historic_future_covariates and future_covariates have the same width
+        input_dim = train_sample[0].shape[1] + (
+            train_sample[1].shape[1] if train_sample[1] is not None else 0
+        )
+        output_dim = train_sample[-1].shape[1]
+        nr_params = 1 if self.likelihood is None else self.likelihood.num_parameters
+
+        if self.rnn_type_or_module in ["RNN", "LSTM", "GRU"]:
+            model = _RNNModule(
+                name=self.rnn_type_or_module,
+                input_size=input_dim,
+                target_size=output_dim,
+                nr_params=nr_params,
+                hidden_dim=self.hidden_dim,
+                dropout=self.dropout,
+                num_layers=self.n_rnn_layers,
+                **self.pl_module_params,
+            )
+        else:
+            model = self.rnn_type_or_module(
+                name="custom_module",
+                input_size=input_dim,
+                target_size=output_dim,
+                nr_params=nr_params,
+                hidden_dim=self.hidden_dim,
+                dropout=self.dropout,
+                num_layers=self.n_rnn_layers,
+                **self.pl_module_params,
+            )
+        return model
+
+    def _build_train_dataset(
+        self,
+        target: Sequence[TimeSeries],
+        past_covariates: Optional[Sequence[TimeSeries]],
+        future_covariates: Optional[Sequence[TimeSeries]],
+        max_samples_per_ts: Optional[int],
+    ) -> DualCovariatesShiftedDataset:
+
+        return DualCovariatesShiftedDataset(
+            target_series=target,
+            covariates=future_covariates,
+            length=self.training_length,
+            shift=1,
+            max_samples_per_ts=max_samples_per_ts,
+        )
+
+    def _verify_train_dataset_type(self, train_dataset: TrainingDataset):
+        raise_if_not(
+            isinstance(train_dataset, DualCovariatesShiftedDataset),
+            "RNNModel requires a training dataset of type DualCovariatesShiftedDataset.",
+        )
+        raise_if_not(
+            train_dataset.ds_past.shift == 1,
+            "RNNModel requires a shifted training dataset with shift=1.",
+        )
