@@ -4,20 +4,20 @@ Theta Method
 """
 
 import math
-from typing import Optional, List
+from typing import List, Optional
 
 import numpy as np
 import statsmodels.tsa.holtwinters as hw
 
+from darts.logging import get_logger, raise_if_not, raise_log
+from darts.models.forecasting.forecasting_model import ForecastingModel
+from darts.timeseries import TimeSeries
 from darts.utils.statistics import (
     check_seasonality,
     extract_trend_and_seasonality,
     remove_from_series,
 )
-from darts.models.forecasting.forecasting_model import ForecastingModel
-from darts.logging import raise_log, get_logger, raise_if_not
-from darts.timeseries import TimeSeries
-from darts.utils.utils import SeasonalityMode, TrendMode, ModelMode
+from darts.utils.utils import ModelMode, SeasonalityMode, TrendMode
 
 logger = get_logger(__name__)
 ALPHA_START = 0.2
@@ -75,7 +75,7 @@ class Theta(ForecastingModel):
 
         raise_if_not(
             season_mode in SeasonalityMode,
-            "Unknown value for season_mode: {}.".format(season_mode),
+            f"Unknown value for season_mode: {season_mode}.",
             logger,
         )
 
@@ -113,12 +113,12 @@ class Theta(ForecastingModel):
             new_ts = remove_from_series(ts, self.seasonality, model=self.season_mode)
 
         # SES part of the decomposition.
-        self.model = hw.SimpleExpSmoothing(new_ts.values()).fit()
+        self.model = hw.SimpleExpSmoothing(new_ts.values(copy=False)).fit()
 
         # Linear Regression part of the decomposition. We select the degree one coefficient.
         b_theta = np.polyfit(
             np.array([i for i in range(0, self.length)]),
-            (1.0 - self.theta) * new_ts.values(),
+            (1.0 - self.theta) * new_ts.values(copy=False),
             1,
         )[0]
 
@@ -127,11 +127,13 @@ class Theta(ForecastingModel):
 
         self.alpha = self.model.params["smoothing_level"]
         if self.alpha == 0.0:
-            self.model = hw.SimpleExpSmoothing(new_ts.values()).fit(
+            self.model = hw.SimpleExpSmoothing(new_ts.values(copy=False)).fit(
                 initial_level=ALPHA_START
             )
             self.alpha = self.model.params["smoothing_level"]
 
+        return self
+
     def predict(self, n: int, num_samples: int = 1) -> "TimeSeries":
         super().predict(n, num_samples)
 
@@ -164,7 +166,7 @@ class Theta(ForecastingModel):
         return self._build_forecast_series(forecast)
 
     def __str__(self):
-        return "Theta({})".format(self.theta)
+        return f"Theta({self.theta})"
 
 
 class FourTheta(ForecastingModel):
@@ -243,17 +245,17 @@ class FourTheta(ForecastingModel):
 
         raise_if_not(
             isinstance(model_mode, ModelMode),
-            "Unknown value for model_mode: {}.".format(model_mode),
+            f"Unknown value for model_mode: {model_mode}.",
             logger,
         )
         raise_if_not(
             isinstance(trend_mode, TrendMode),
-            "Unknown value for trend_mode: {}.".format(trend_mode),
+            f"Unknown value for trend_mode: {trend_mode}.",
             logger,
         )
         raise_if_not(
             isinstance(season_mode, SeasonalityMode),
-            "Unknown value for season_mode: {}.".format(season_mode),
+            f"Unknown value for season_mode: {season_mode}.",
             logger,
         )
 
@@ -263,7 +265,7 @@ class FourTheta(ForecastingModel):
         self.length = len(series)
         # normalization of data
         if self.normalization:
-            self.mean = series.mean().mean()
+            self.mean = series.pd_dataframe(copy=False).mean().mean()
             raise_if_not(
                 not np.isclose(self.mean, 0),
                 "The mean value of the provided series is too close to zero to perform normalization",
@@ -339,14 +341,16 @@ class FourTheta(ForecastingModel):
             self.fitted_values = self.wses * theta2_in + self.wdrift * theta0_in
         if self.is_seasonal:
             if self.season_mode is SeasonalityMode.ADDITIVE:
-                self.fitted_values += self.seasonality.univariate_values()
+                self.fitted_values += self.seasonality.univariate_values(copy=False)
             elif self.season_mode is SeasonalityMode.MULTIPLICATIVE:
-                self.fitted_values *= self.seasonality.univariate_values()
+                self.fitted_values *= self.seasonality.univariate_values(copy=False)
         # Fitted values are the results of the fit of the model on the train series. A good fit of the model
         # will lead to fitted_values similar to ts. But one cannot see if it overfits.
         if self.normalization:
             self.fitted_values *= self.mean
 
+        return self
+
     def predict(self, n: int, num_samples: int = 1) -> "TimeSeries":
         super().predict(n, num_samples)
 
@@ -420,7 +424,7 @@ class FourTheta(ForecastingModel):
 
         if thetas is None:
             thetas = [1, 2, 3]
-        if (ts.values() <= 0).any():
+        if (ts.values(copy=False) <= 0).any():
             drift_mode = [TrendMode.LINEAR]
             model_mode = [ModelMode.ADDITIVE]
             season_mode = [SeasonalityMode.ADDITIVE]
