@@ -30,36 +30,33 @@ errors will be raised during training. You can refer to the individual likelihoo
 to see what is the support. Similarly, the prior parameters also have to lie in some pre-defined domains.
 """
 
-# TODO: Table on README listing distribution, possible priors and wiki article
-from darts.utils.utils import raise_if_not
-
-from abc import ABC, abstractmethod
-from typing import Optional, Tuple, List
 import collections
+from abc import ABC, abstractmethod
+from typing import List, Optional, Tuple
 
 import numpy as np
 import torch
 import torch.nn as nn
+from torch.distributions import Bernoulli as _Bernoulli
+from torch.distributions import Beta as _Beta
+from torch.distributions import Cauchy as _Cauchy
+from torch.distributions import ContinuousBernoulli as _ContinuousBernoulli
+from torch.distributions import Dirichlet as _Dirichlet
+from torch.distributions import Exponential as _Exponential
+from torch.distributions import Gamma as _Gamma
+from torch.distributions import Geometric as _Geometric
+from torch.distributions import Gumbel as _Gumbel
+from torch.distributions import HalfNormal as _HalfNormal
+from torch.distributions import Laplace as _Laplace
+from torch.distributions import LogNormal as _LogNormal
+from torch.distributions import NegativeBinomial as _NegativeBinomial
+from torch.distributions import Normal as _Normal
+from torch.distributions import Poisson as _Poisson
+from torch.distributions import Weibull as _Weibull
 from torch.distributions.kl import kl_divergence
-from torch.distributions import (
-    Normal as _Normal,
-    Poisson as _Poisson,
-    NegativeBinomial as _NegativeBinomial,
-    Bernoulli as _Bernoulli,
-    Gamma as _Gamma,
-    Gumbel as _Gumbel,
-    Laplace as _Laplace,
-    Beta as _Beta,
-    Exponential as _Exponential,
-    MultivariateNormal as _MultivariateNormal,
-    Dirichlet as _Dirichlet,
-    Geometric as _Geometric,
-    Cauchy as _Cauchy,
-    ContinuousBernoulli as _ContinuousBernoulli,
-    HalfNormal as _HalfNormal,
-    LogNormal as _LogNormal,
-    Weibull as _Weibull,
-)
+
+# TODO: Table on README listing distribution, possible priors and wiki article
+from darts.utils.utils import raise_if_not
 
 MIN_CAUCHY_GAMMA_SAMPLING = 1e-100
 
@@ -71,12 +68,12 @@ def _check(param, predicate, param_name, condition_str):
     if isinstance(param, (collections.Sequence, np.ndarray)):
         raise_if_not(
             all(predicate(p) for p in param),
-            "All provided parameters {} must be {}.".format(param_name, condition_str),
+            f"All provided parameters {param_name} must be {condition_str}.",
         )
     else:
         raise_if_not(
             predicate(param),
-            "The parameter {} must be {}.".format(param_name, condition_str),
+            f"The parameter {param_name} must be {condition_str}.",
         )
 
 
@@ -111,13 +108,11 @@ class Likelihood(ABC):
             out_distr = self._distr_from_params(params_out)
             device = params_out[0].device
             prior_params = tuple(
-                [
-                    # use model output as "prior" for parameters not specified as prior
-                    torch.tensor(prior_params[i]).to(device)
-                    if prior_params[i] is not None
-                    else params_out[i]
-                    for i in range(len(prior_params))
-                ]
+                # use model output as "prior" for parameters not specified as prior
+                torch.tensor(prior_params[i]).to(device)
+                if prior_params[i] is not None
+                else params_out[i]
+                for i in range(len(prior_params))
             )
             prior_distr = self._distr_from_params(prior_params)
 
@@ -185,8 +180,8 @@ class GaussianLikelihood(Likelihood):
         https://en.wikipedia.org/wiki/Normal_distribution
 
         - Univariate continuous distribution.
-        - Support: :math:`\mathbb{R}`.
-        - Parameters: mean :math:`\\mu \in \mathbb{R}`, standard deviation :math:`\\sigma > 0`.
+        - Support: :math:`\\mathbb{R}`.
+        - Parameters: mean :math:`\\mu \\in \\mathbb{R}`, standard deviation :math:`\\sigma > 0`.
 
         Parameters
         ----------
@@ -243,7 +238,7 @@ class PoissonLikelihood(Likelihood):
         https://en.wikipedia.org/wiki/Poisson_distribution
 
         - Univariate discrete distribution
-        - Support: :math:`\mathbb{N}_0` (natural numbers including 0).
+        - Support: :math:`\\mathbb{N}_0` (natural numbers including 0).
         - Parameter: rate :math:`\\lambda > 0`.
 
         Parameters
@@ -295,8 +290,8 @@ class NegativeBinomialLikelihood(Likelihood):
         It does not support priors.
 
         - Univariate discrete distribution.
-        - Support: :math:`\mathbb{N}_0` (natural numbers including 0).
-        - Parameters: number of failures :math:`r > 0`, success probability :math:`p \in (0, 1)`.
+        - Support: :math:`\\mathbb{N}_0` (natural numbers including 0).
+        - Parameters: number of failures :math:`r > 0`, success probability :math:`p \\in (0, 1)`.
 
         Behind the scenes the distribution is reparameterized so that the actual outputs of the
         network are in terms of the mean :math:`\\mu` and shape :math:`\\alpha`.
@@ -344,8 +339,8 @@ class BernoulliLikelihood(Likelihood):
         https://en.wikipedia.org/wiki/Bernoulli_distribution
 
         - Univariate discrete distribution.
-        - Support: :math:`\{0, 1\}`.
-        - Parameter: probability :math:`p \in (0, 1)`.
+        - Support: :math:`\\{0, 1\\}`.
+        - Parameter: probability :math:`p \\in (0, 1)`.
 
         Parameters
         ----------
@@ -440,8 +435,8 @@ class CauchyLikelihood(Likelihood):
         https://en.wikipedia.org/wiki/Cauchy_distribution
 
         - Univariate continuous distribution.
-        - Support: :math:`\mathbb{R}`.
-        - Parameters: location :math:`x_0 \in \mathbb{R}`, scale :math:`\gamma > 0`.
+        - Support: :math:`\\mathbb{R}`.
+        - Parameters: location :math:`x_0 \\in \\mathbb{R}`, scale :math:`\\gamma > 0`.
 
         Due to its fat tails, this distribution is typically harder to estimate,
         and your mileage may vary. Also be aware that it typically
@@ -499,7 +494,7 @@ class ContinuousBernoulliLikelihood(Likelihood):
 
         - Univariate continuous distribution.
         - Support: open interval :math:`(0, 1)`.
-        - Parameter: shape :math:`\\lambda \in (0,1)`
+        - Parameter: shape :math:`\\lambda \\in (0,1)`
 
         Parameters
         ----------
@@ -545,7 +540,7 @@ class DirichletLikelihood(Likelihood):
 
         - Multivariate continuous distribution, modeling all components of a time series jointly.
         - Support: The :math:`K`-dimensional simplex for series of dimension :math:`K`, i.e.,
-          :math:`x_1, ..., x_K \\text{ with } x_i \in (0,1),\\; \\sum_i^K{x_i}=1`.
+          :math:`x_1, ..., x_K \\text{ with } x_i \\in (0,1),\\; \\sum_i^K{x_i}=1`.
         - Parameter: concentrations :math:`\\alpha_1, ..., \\alpha_K` with :math:`\\alpha_i > 0`.
 
         Parameters
@@ -592,7 +587,7 @@ class ExponentialLikelihood(Likelihood):
         https://en.wikipedia.org/wiki/Exponential_distribution
 
         - Univariate continuous distribution.
-        - Support: :math:`\mathbb{R}_{>0}`.
+        - Support: :math:`\\mathbb{R}_{>0}`.
         - Parameter: rate :math:`\\lambda > 0`.
 
         Parameters
@@ -637,7 +632,7 @@ class GammaLikelihood(Likelihood):
         https://en.wikipedia.org/wiki/Gamma_distribution
 
         - Univariate continuous distribution
-        - Support: :math:`\mathbb{R}_{>0}`.
+        - Support: :math:`\\mathbb{R}_{>0}`.
         - Parameters: shape :math:`\\alpha > 0` and rate :math:`\\beta > 0`.
 
         Parameters
@@ -687,8 +682,8 @@ class GeometricLikelihood(Likelihood):
         https://en.wikipedia.org/wiki/Geometric_distribution
 
         - Univariate discrete distribution
-        - Support: :math:`\mathbb{N}_0` (natural numbers including 0).
-        - Parameter: success probability :math:`p \in (0, 1)`.
+        - Support: :math:`\\mathbb{N}_0` (natural numbers including 0).
+        - Parameter: success probability :math:`p \\in (0, 1)`.
 
         Parameters
         ----------
@@ -732,8 +727,8 @@ class GumbelLikelihood(Likelihood):
         https://en.wikipedia.org/wiki/Gumbel_distribution
 
         - Univariate continuous distribution
-        - Support: :math:`\mathbb{R}`.
-        - Parameters: location :math:`\\mu \in \mathbb{R}` and scale :math:`\\beta > 0`.
+        - Support: :math:`\\mathbb{R}`.
+        - Parameters: location :math:`\\mu \\in \\mathbb{R}` and scale :math:`\\beta > 0`.
 
         Parameters
         ----------
@@ -781,7 +776,7 @@ class HalfNormalLikelihood(Likelihood):
         https://en.wikipedia.org/wiki/Half-normal_distribution
 
         - Univariate continuous distribution.
-        - Support: :math:`\mathbb{R}_{>0}`.
+        - Support: :math:`\\mathbb{R}_{>0}`.
         - Parameter: rate :math:`\\sigma > 0`.
 
         Parameters
@@ -826,8 +821,8 @@ class LaplaceLikelihood(Likelihood):
         https://en.wikipedia.org/wiki/Laplace_distribution
 
         - Univariate continuous distribution
-        - Support: :math:`\mathbb{R}`.
-        - Parameters: location :math:`\\mu \in \mathbb{R}` and scale :math:`b > 0`.
+        - Support: :math:`\\mathbb{R}`.
+        - Parameters: location :math:`\\mu \\in \\mathbb{R}` and scale :math:`b > 0`.
 
         Parameters
         ----------
@@ -875,8 +870,8 @@ class LogNormalLikelihood(Likelihood):
         https://en.wikipedia.org/wiki/Log-normal_distribution
 
         - Univariate continuous distribution.
-        - Support: :math:`\mathbb{R}_{>0}`.
-        - Parameters: :math:`\\mu \in \mathbb{R}` and :math:`\\sigma > 0`.
+        - Support: :math:`\\mathbb{R}_{>0}`.
+        - Parameters: :math:`\\mu \\in \\mathbb{R}` and :math:`\\sigma > 0`.
 
         Parameters
         ----------
@@ -924,7 +919,7 @@ class WeibullLikelihood(Likelihood):
         https://en.wikipedia.org/wiki/Weibull_distribution
 
         - Univariate continuous distribution
-        - Support: :math:`\mathbb{R}_{>0}`.
+        - Support: :math:`\\mathbb{R}_{>0}`.
         - Parameters: scale :math:`\\lambda > 0` and concentration :math:`k > 0`.
 
         It does not support priors.
@@ -1108,12 +1103,11 @@ class QuantileRegression(Likelihood):
         return None
 
 
-if False:
-    """TODO
-    To make it work, we'll have to change our models so they optionally accept an absolute
-    number of parameters, instead of num_parameters per component.
-    """
+""" TODO
+To make it work, we'll have to change our models so they optionally accept an absolute
+number of parameters, instead of num_parameters per component.
 
+from torch.distributions import MultivariateNormal as _MultivariateNormal
     class MultivariateNormal(Likelihood):
         def __init__(
             self, dim: int, prior_mu=None, prior_covmat=None, prior_strength=1.0
@@ -1172,3 +1166,4 @@ if False:
             covmat[tril_indices[0], tril_indices[1]] = covmat_coefs
             covmat[tril_indices[1], tril_indices[0]] = covmat_coefs
             covmat[range(self.dim), range(self.dim)] = 1.0
+"""
